#First load some necessary stuff:
LoadPackage("quagroup");;
Read("kostantbasis.g");
Read("sparse.g");

echelonmat:= function ( mat )
    local  ech, tmp, i;
    if Length( mat ) = 0  then
        return mat;
    fi;
    ech := SemiEchelonMatDestructive( mat );
    tmp := [  ];
    for i  in [ 1 .. Length( ech.heads ) ]  do
        if ech.heads[i] <> 0  then
            Add( tmp, ech.vectors[ech.heads[i]] );
        fi;
    od;
    return tmp;
end;

InitialData:=function(g,hw,F)
	#Here g is a semisimple Lie algebra over the Rationals, 
	#hw a dominant weight, and F a field. 
	#This function will return a record with the following fields:
	#
	#First some fields over the Rationals:
	#
	#V:	the irreducible module of highest weight hw over Rationals
	#BV:	a Kostant (??) basis of V
	#v0:	the highest weight vector of V
	#n:	dim(V)
	#u:	the Lie algebra of U_ (see documentation)
	#r:	dim(U_)
	#
	#Then some values dependent on the field:
	#
	#F:	the field itself
	#Bv0:	v0 as a row relative to BV, over F
	#Bu:	a list of lists of matrices over F: us[i] contains the
	#	nonzero divided powers of the i-th element of a Chevalley 
	#	basis of u_, as a matrix over F relative to BV
	#TC:	a list of rows spanning the tangent space to C at v0, 
	#	by rows over F relative to BV.
	#
	#And finally some fields containing the results so far:
	#
	#k: 	secant dimensions have been computed up to k
	#v: 	a list of the k elements of V used so far
	#T: 	a list of length k; T[i] is a basis of the tangent
	#	space to kC at the sum of the v[i].
	#dim:	a list of length k; dim[i] is the 
	#	sum of the tangent spaces at v[j] with j<=i.

	local V,BV,v0,n,R,X,Y,L,x,y,u,i,Bv0,TC,set,size,sparse,fam, t, idmat;

	#First we construct the module and its highest-weight vector:
#t:= Runtime();
	V:=HighestWeightModule(g,hw);
#Print("hwmod: ",Runtime()-t," dim ",Dimension(V),"\n"); t:= Runtime();
	BV:=KostantBasis(V);
#Print("Kostant:  ",Runtime()-t,"\n");
	v0:=BV[1];
	n:=Dimension(V);

	#Then we compute u:
	R:=RootSystem(g);
	X:=NegativeRootVectors(R);
	Y:=[]; 
	for x in X do 
		if x^v0 <> Zero(V) then Append(Y,[x]); fi; 
	od;
	u:=Subalgebra(g,Y);
#t:= Runtime();
	#The divided powers of the Chevalley basis of u_:
        if n <= 200 then
	   Y:=List(Y,y->MatrixOfAction(BV,y));
           sparse:= false;
        else
           fam:= SparseMatFam( F, n, n );
           Y:=List(Y,y->SparseMatrix(fam,MatrixOfAction(BV,y)*One(F)));
           sparse:= true;
        fi;

	X:=[];
	for y in Y do 
		i:=1;
		x:=y;
		L:=[];
		while not IsZero(x) do
			Add(L,x);
			i:=i+1;
			x:=x*y/i;
		od;
		Append(X,[L]);
	od;
#Print("mats:  ",Runtime()-t,"\n");
	#The tangent space at v0:
	Bv0:=Coefficients(BV,v0)*One(F);
	TC:=Concatenation([Bv0], List(X,m->m[1]*Bv0));

        if sparse then
           idmat:= SparseMatrix( FamilyObj( X[1][1] ), IdentityMat(n,F) );
        else 
     	   idmat:=IdentityMat(n,F);
        fi;

	return rec(
		V:=V,
		BV:=BV,
		v0:=v0,
		n:=n,
		u:=u,
		r:=Dimension(u),
		F:=F,
		Bv0:=Bv0,
		Bu:=X,
		TC:=TC,
		# 
		k:=1,
		v:=[Bv0],
		#T:=[echelonmat(TC)],
                Tk:= echelonmat(TC), 
		dim:=[Length(TC)],
                sparse:= sparse,
                idmat:= idmat
	);
end;


RandomU:=function( R, set )
#given the record R, this function computes a random element of the
#Chevalley group.
	local x,y,z,l,s,t,time;

#        if R.sparse then
#           x:= SparseMatrix( FamilyObj( R.Bu[1][1] ), IdentityMat(R.n,R.F) );
#        else 
#     	   x:=IdentityMat(R.n,R.F);
#        fi;
        x:= R.idmat;

	for l in R.Bu do

#            if R.sparse then
#               y:= SparseMatrix( FamilyObj( R.Bu[1][1] ), IdentityMat(R.n,R.F) );
#            else 
#               y:=IdentityMat(R.n,R.F);
#            fi;
            y:= R.idmat;
            t:= Random( set );
            s:=One(R.F);
	    for z in l do
		s:=t*s;
		y:=y+s*z;
	    od;
	    #Now y equals exp(t*l[1]).
	    x:=x*y;	
	od;
	return x;
end;

#etime:= 0;

SecantDimension:=function(l,R)
#This function computes the secant dimensions up to the l-th one and
#stores the result in R.
	local i,U,vl,vv,set,noits,done, t;
	i:=R.k;

	while i<l do 

        noits:= 1; done:= false;
        while noits <= 5 and not done do
              if noits = 5 then # final iteration: whole field
                 set:= R.F;
              else
                 set:= List( [0..2*noits-1], x -> x*One(R.F) );;
              fi;
     	      U:=RandomU(R,set);
              vl:=U*R.Bv0;
              vv:= List( R.Tk, ShallowCopy );
              Append( vv, List(R.TC,w->U*w) );
#t:= Runtime();
              vv:= echelonmat( vv );
#etime:= etime+Runtime()-t;
              if Length(vv) = (i+1)*Length(R.TC) or Length(vv) = R.n then
                 done:= true;
              fi;
              noits:= noits+1;
        od;
		Append(R.v,[vl]);
        #Add( R.T, vv );
        R.Tk:= vv;
	    #Append(R.T,[EcheloniseMat(Concatenation(R.T[i],
		# 		List(R.TC,w->U*w)))]);
        Add(R.dim, Length(vv) );
  		i:=i+1;
	od;
	R.k:=i;

	return R.dim[l];
end;

Discard:=function(l,R)
#This function throws away all results from l+1 on.
	R.k:=Maximum(1,l);
	R.v:=R.v{[1..R.k]};
	#R.T:=R.T{[1..R.k]};
	R.dim:=R.dim{[1..R.k]};
end;

IsDefective:=function(R)
#This function checks whether the secant varieties computed until now
#are defective.
	local b,i;
	b:=false;
	i:=1;
	while not b and i<=R.k do 
		b:=R.dim[i]<i*R.dim[1] and R.dim[i]<R.n;
		i:=i+1;
	od;
	return b;
end;
