DeclareCategory( "IsSparseMatrix", IsMatrix );
DeclareCategoryFamily( "IsSparseMatrix" );

SparseMatFam:= function( F, m, n )

       local fam, type;

       fam:= NewFamily( "SparseMatFam", IsSparseMatrix );
       fam!.noRows:= m; fam!.noCols:= n; fam!.field:= F;
       type:= NewType( fam, IsMatrix and IsSparseMatrix and IsComponentObjectRep );
       fam!.spmatType:= type;
       return fam;

end;

InstallMethod( ObjByExtRep,
   "for family of sparse matrices, and matrix",
   true, [ IsSparseMatrixFamily, IsList ], 0,
   function( fam, m )

    return Objectify( fam!.spmatType,
                    rec( mat:= m ) );
end );

SparseMatrix:= function( fam, m )

    local sp, i, j, l, r;

    sp:= [ ];
    for i in [1..Length(m)] do
        l:= [ [], [] ];
        r:= m[i];
        for j in [1..Length(r)] do
            if not IsZero( r[j] ) then
               Add( l[1], j ); Add( l[2], r[j] );
            fi;
        od;
        Add( sp, l );
    od;
    return ObjByExtRep( fam, Immutable(sp) );
end;

DenseMatrix:= function( m )

    return List( [1..FamilyObj(m)!.noRows], x -> m[x] );

end; 

InstallMethod( \[\],
    "for a sparse matrix and an integer",
    true, [ IsSparseMatrix, IsPosInt ], 0,
    function( m, i )

    local res, c, z, row, down, k, j;

    res:= [ ];
    c:= FamilyObj( m )!.noCols;
    z:= Zero( FamilyObj(m)!.field );
    row:= m!.mat[i];
    down:= 1;
    for k in [1..Length(row[1])] do
        for j in [down..row[1][k]-1] do
            Add( res, z );
        od;
        Add( res, row[2][k] );
        down:= row[1][k]+1;
    od;
    for j in [down..c] do Add( res, z ); od;

    return res;

end );

InstallMethod( PrintObj,
    "for a sparse matrix",
    true, [ IsSparseMatrix ], 0,
    function( m )
    local f;
    f:= FamilyObj( m );
    Print("<a ",f!.noRows,"x",f!.noCols,"-sparse matrix>");

end );

insert:= function( row, row0, cf )

     local i, p;

     for i in [1..Length(row0[1])] do
         p:= PositionSorted( row[1], row0[1][i] );
         if p > Length( row[1] ) then
            Add( row[1], row0[1][i] );
            Add( row[2], cf*row0[2][i] );
         elif row[1][p] = row0[1][i] then
            row[2][p]:= row[2][p] + cf*row0[2][i];
            if IsZero( row[2][p] ) then
               Unbind( row[2][p] ); Unbind( row[1][p] );
               row[1]:= Filtered( row[1], x -> IsBound(x) );
               row[2]:= Filtered( row[2], x -> IsBound(x) );
            fi; 
         else
            InsertElmList( row[1], p, row0[1][i] );
            InsertElmList( row[2], p, cf*row0[2][i] );
         fi;
     od;

end;

InstallMethod( \*,
    "for two sparse matrices",
    IsIdenticalObj, [ IsSparseMatrix, IsSparseMatrix ], 0,
    function( m1, m2 )

    local fam, n, m, a, b, new, k, l, i, r, elm, ind, pos;

    #assume both are square
    fam:= FamilyObj( m1 );
    n:= fam!.noRows;
    m:= [ ];
    a:= m1!.mat; b:= m2!.mat;
    for k in [1..n] do
        r:= a[k];
        if Length(r[1]) > 0 then
           ind:= r[1][1];
           new:= [ ShallowCopy( b[ind][1] ), r[2][1]*ShallowCopy( b[ind][2] ) ];
           for l in [2..Length(r[1])] do
               insert( new, b[r[1][l]], r[2][l] );
           od;
        else
           new:= [ [], [] ];
        fi;
        Add( m, new );
    od;

    return ObjByExtRep( fam, m );
                            
end );
    
InstallMethod( \+,
    "for two sparse matrices",
    IsIdenticalObj, [ IsSparseMatrix, IsSparseMatrix ], 0,
    function( m1, m2 )

    local fam, n, m, a, b, new, k;

    fam:= FamilyObj( m1 );
    n:= fam!.noRows;
    m:= [ ];
    a:= m1!.mat; b:= m2!.mat;
    for k in [1..n] do
        new:= List( a[k], ShallowCopy );
        insert( new, b[k], One(fam!.field) );
        Add( m, new );
    od;

    return ObjByExtRep( fam, m );
                            
end );

InstallMethod( \*,
    "for a sparse matrix and a vector",
    true, [ IsSparseMatrix, IsVector ], 0,
    function( m, v )

    local w, a, fam, i, j, x;

    w:= [ ];
    a:= m!.mat;
    fam:= FamilyObj(m);
    for i in [1..fam!.noRows] do
        x:= Zero( fam!.field );
        for j in [1..Length(a[i][1])] do
            x:= x + a[i][2][j]*v[ a[i][1][j] ];
        od;
        Add( w, x );
    od;
    return w;

end );


InstallMethod( \*,
    "for a sparse matrix and a scalar",
    true, [ IsSparseMatrix, IsScalar ], 0,
    function( m, x )
      if IsZero(x) then 
         return ObjByExtRep( FamilyObj(m), List( m!.mat, u -> [ [], [] ] ) );
      else
         return ObjByExtRep( FamilyObj(m), List( m!.mat, u -> [ ShallowCopy( u[1] ), x*ShallowCopy( u[2] ) ] ) );
      fi;
end);

InstallMethod( \*,
    "for a sparse matrix and a scalar",
    true, [ IsScalar, IsSparseMatrix ], 0,
    function( x, m)
      if IsZero(x) then 
         return ObjByExtRep( FamilyObj(m), List( m!.mat, u -> [ [], [] ] ) );
      else
         return ObjByExtRep( FamilyObj(m), List( m!.mat, u -> [ ShallowCopy( u[1] ), x*ShallowCopy( u[2] ) ] ) );
      fi;
end);

InstallMethod( IsZero,
    "for a sparse matrix",
    true, [ IsSparseMatrix ], 0,
    function( m )
      return ForAll( List( m!.mat, u -> Length(u[1]) ), b -> b=0 );
end);

InstallMethod( ZERO,
    "for a sparse matrix",
    true, [ IsSparseMatrix], 0,
    function( m)
      return ObjByExtRep( FamilyObj(m), List( m!.mat, u -> [ [], [] ] ) );
end);

#InstallMethod( ONE,
#    "for a sparse matrix",
#    true, [ IsSparseMatrix ], 0,
#    function( m)
#      return ObjByExtRep( FamilyObj(m), List( [1..FamilyObj(m)!.noCols], 
#                                            u -> [ [u], [One(FamilyObj(m)!.field)] ] ) );
#end);


